// accumurating for 1 second and sending data to clients

#ifndef _THREAD_CH_DCON_
#define _THREAD_CH_DCON_

#include <vector>
using namespace std;

extern "C" {
#include <signal.h>
#include <wchar.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <math.h>
}

#include "Thread.h"
#include "Mutex.h"
#include "FileSocket.h"
#include "TCPSocket.h"
#include "payload.h"
#include "netutl.h"

extern char* me;
extern int debug_level;

#define SHM_DATA ftok("/tmp",1)
#define SHM_PNTR ftok("/tmp",2)
const long nano_wait= 5;

enum State {NORMAL_STATE,WARN_STATE,STOP_STATE,ACTIVE_STATE};
const int PAGE_READY_WAIT=10;
const int SOCK_READY_WAIT=10;

//inline int delete_shm(const int shm_id);

class Recv_mem_simple: public Thread
{
  Payload payld;
  int ipage; 
  u_int8_t* page[2]; 
  u_int8_t* w_point; // 
  u_int8_t* r_point; // 
  Mutex  page_mutex[2];
  char page_state[2];  // 0:write OK, 1: read OK 
  unsigned long recvd_bytes[2]; // recived bytes for each pages
  int mtu;
  Socket* _in_sock;
  int appl_status; // communicator with application
  int so_rBufSize; 
protected:
  string srv_ip;
  int srv_port;
  int mem_data_id;
  unsigned long mem_data_size; // memory size 
  static int nClientID;
public:
  //  Recv_mem_simple( Socket& out_sock);
  Recv_mem_simple(const int m );
  ~Recv_mem_simple(){  Release_data_mem();  }
  const int Get_ipage()const {return ipage;}
  //  u_int8_t* Get_Rpoint() const {return r_point;}
  const u_int8_t* Get_Wpoint() const {return w_point;}
  const u_int32_t Get_Pagesize() const {return mem_data_size/2;}
  u_int8_t* Get_page_addr(const int i)const{return page[i];}
  const int Check_forRead(const int i);  
  const int Check_forWrite(const int i);  
  const int Read_finsh(const int i);
  const int Write_finsh(const int i);
  //  void Inc_Rpoint(const u_int32_t d);
  //  void Set_insock(Socket* s){_in_sock=s;}
  void Set_payload(const struct Payload& pld){  payld = pld; }
  //  void Set_RecvBufSize(const u_int32_t bs){recv_nword = bs/4;} // sps
  void Set_server(const string ip){srv_ip = ip;}
  void Set_Srv_port(const int p){srv_port=p;}
  void Print(FILE* fp=stderr);
  int  Setup_data_mem();
  //  int  Delete_shm();
  //  int  Release_data_mem(){if( page[0] !=0){ delete [] page[0]; page[0]=0;}}
  const int  Release_data_mem();
  const int CommunicateWithServer(unsigned long sec,  
				  const string& srv_ip, const int port);
  void* thread_func(void);
  void proc();
  const int Get_shmid()const {return mem_data_id;}
  const unsigned long Get_RecvdByte(const int ipage){return recvd_bytes[ipage];}
  const bool Check_ApplState(const State s){return (appl_status == s);}
  void Set_ApplState(const State s){appl_status = s;}
  void Set_so_rBufSize(const int& sb){so_rBufSize =sb;}
  //  const int Send_payload();
};
int Recv_mem_simple::nClientID = 0;


// Constructor 
inline Recv_mem_simple::Recv_mem_simple(const int m):
  mem_data_size(0), mem_data_id(0),mtu(m),_in_sock(0),
     //  recv_nword(256), 
     w_point(0),r_point(0),ipage(0),
     srv_port(7002),srv_ip("127.0.0.1"),appl_status(NORMAL_STATE)
{
  page[0] = page[1] =0;
  page_state[0] = page_state[1] =0; // write OK;
}

inline  void Recv_mem_simple::proc(){
  for(int i=0; i<2; i++){
    page_mutex[i].lock(); // wait until data is ready
    page_state[i] =0;
    page_mutex[i].unlock();
  }

  if(nClientID ==0) start();
  else fprintf(stderr,"Error: Receiver is already running\n");
}

inline const int Recv_mem_simple::Check_forRead(const int i){
  struct timespec req={0,nano_wait}; // 5 nano sec
  
  if( i != 0 && i !=1){
    fprintf(stderr,"Check_forRead(): Error %d not 0 or 1\n", i);
    return -1;
  }
  int scnt=0, pcnt=0;
  while(scnt < SOCK_READY_WAIT && 
	pcnt < PAGE_READY_WAIT ){
//     if( _in_sock==0){
// 	usleep(1);
// 	scnt++;
//  	continue;
//       }
      if( page[0]==0){
	usleep(1);
	pcnt++;
 	continue;
      }
//       if(_in_sock->select(Socket::SEL_EXCEPT,1.e-6)){
// 	fprintf(stderr,"Exception _in_sock, probably closed.\n");
// 	return -2;
//       }
      if( page_state[i]==1 || Check_ApplState(STOP_STATE)) 	break; // read OK
      nanosleep(&req,0);
      if(debug_level > 1)
	fprintf(stderr,"\rNot ready for read(%d) %d %d", 
		i, page_state[0], page_state[1]);
    }
  if(scnt >= SOCK_READY_WAIT){
    fprintf(stderr,
	    "Check_forRead:Too much waiting(%d us) for Socket for data source\n", 
	    SOCK_READY_WAIT);
    return -1;
  }
  if(pcnt >= PAGE_READY_WAIT){
    fprintf(stderr,"Check_forRead:Too much waiting (%d us) for Memory for data source\n",
	    PAGE_READY_WAIT);
    return -1;
  }
  if(Check_ApplState(STOP_STATE)){
    fprintf(stderr,"Check_forWrite:Announce 'Data Reading Application stopped' received,(state=%d)\n",
	    appl_status);
    fprintf(stderr,"quit to avoid dead lock\n");
    return -1;
  }

  if( debug_level > 0)
    fprintf(stderr,"Page(%d) Ready for Read...",i);
  return 0;


}

inline const int Recv_mem_simple::Check_forWrite(const int i){
  struct timespec req={0,nano_wait}; // 5 nano sec
 
  if( i != 0 && i !=1){
    fprintf(stderr,"Check_forWrite(): Error %d not 0 or 1\n", i);
    return -1;
  }

  int scnt=0, pcnt=0;
  while(scnt < SOCK_READY_WAIT && 
	pcnt < PAGE_READY_WAIT ){
    if( _in_sock==0){
      usleep(1);
      scnt++;
      continue;
    }
    if( page[0]==0){
      usleep(1);
      pcnt++;
      continue;
    }
    if(_in_sock->select(Socket::SEL_EXCEPT,1.e-6)){
      fprintf(stderr,"Exception _in_sock, probably closed.\n");
      return -2;
    }
    if( page_state[i]==0 || Check_ApplState(STOP_STATE))break; // if writable
    nanosleep(&req,0);
    if(debug_level > 1)
      fprintf(stderr,"\rNot ready for write(%d)", i);
    //      else fprintf(stderr,"page(%d) is still locked\n",i);
  }
  
  if(scnt >= SOCK_READY_WAIT){
    fprintf(stderr,"Check_forWrite:Too much waiting(%d us) for Socket for data source\n", 
	    SOCK_READY_WAIT);
    return -1;
  }
  if(pcnt >= PAGE_READY_WAIT){
    fprintf(stderr,"Check_forWrite:Too much waiting (%d us) for Memory for data source\n",
	    PAGE_READY_WAIT);
    return -1;
  }

  if(Check_ApplState(STOP_STATE)){
    fprintf(stderr,"Check_forWrite:Announce 'Data Reading Application stopped' received,(state=%d)\n",
	    appl_status);
    fprintf(stderr,"quit to avoid dead lock\n");
    return -1;
  }
  
  if( debug_level > 0)
    fprintf(stderr,"Page(%d) Ready for write...",i);
  return 0;
}


inline const int Recv_mem_simple::Read_finsh(const int i){
  if( i == 0 || i ==1){
    page_mutex[i].lock();
    page_state[i]=0; //writable
    page_mutex[i].unlock();
    return 0;
  }else{
    fprintf(stderr,"Read_finish(): Error %d not 0 or 1\n", i);
    return -1;
  }
}

inline const int Recv_mem_simple::Write_finsh(const int i){
  if( i == 0 || i ==1){
    page_mutex[i].lock();
    page_state[i]=1; //readable
    page_mutex[i].unlock();
    return 0;
  }else{
    fprintf(stderr,"Write_finish(): Error %d not 0 or 1\n", i);
    return -1;
  }
}

inline void Recv_mem_simple::Print(FILE *fp){
  _in_sock->Print(fp);
  fprintf(fp," Config of Recv_mem_simple data------------\n");
  fprintf(fp, "Memory size=%ld MB\n", mem_data_size/1000000);
  fprintf(fp, "Srv_ip=%s:%d \n", srv_ip.c_str(), srv_port);
  payld.Print(fp);
  fprintf(fp," -------------------------------------\n");
}

// Get Shared memory for data
inline int Recv_mem_simple::Setup_data_mem(){
  // for 2 pages for in and out
  mem_data_size= payld.clock/8*2* payld.Nbits(); // in Byte for 2pages 
  if( page[0] !=0) delete page[0];
  page[0] = new u_int8_t[mem_data_size];
  // Pointer initialize 
  page[1] = page[0] + mem_data_size/2;
  r_point = w_point = page[0];
  fprintf(stderr,"Memory(%.3f MB 2 pages) successfully allocated\n",
	  (float)mem_data_size/2000000.);
  // fprintf(stderr,"Shared memID(%ld MB 2 pages) = %d successed\n",  mem_data_size/2000000, mem_data_id);
  return 0;
}

const int Recv_mem_simple::Release_data_mem(){
  
//   int ret = shmdt(page[0]);
//   if(ret==0) fprintf(stderr,"Shared Memory (id=%d, size=%.3f MB) detached\n",  mem_data_id, (float)mem_data_size*1.e-6);
  if( page[0] !=0){ 
    delete page[0];
    fprintf(stderr,"Memory Area (%.3f MB) released\n", (float)mem_data_size*1.e-6);
  }
  page[0] = page[1] = w_point = r_point = 0;
  mem_data_size=0;
  return 0;
}


// Communications with Client 
inline const int Recv_mem_simple::CommunicateWithServer(unsigned long sec, 
					     const string& srv_ip,
					     const int port){
  
  long long byte_sec = mem_data_size/2; // Byte for 1sec. 
  int status;
  u_int8_t rBuf[100000];
  
  fprintf(stderr,"DEBUG: CommunicateWithServer Started\n");   
  signal(SIGTSTP, SIG_IGN);

  _in_sock = new TCPSocket(mtu);
  // Bind
/*   char name[80]; */
/*   gethostname(name,80); */
/*   if(_in_sock->bind( name,0) == -1){ */
/*     fprintf(stderr,"%s _in_sock->bind(): Error\n",me); */
/*     perror("sock->bind"); */
/*     Set_ApplState(STOP_STATE); */
/*      if(_in_sock->get_sockd() != -1) _in_sock->close(); */
/*      delete _in_sock; */
/*      _in_sock=0; */
/*      return -1; */
/*   } */

  _in_sock->set_so_rcvbuf(so_rBufSize);
  Print();

  fprintf(stderr, "Connection to server ...");
  if( _in_sock->connect(srv_ip, port) == -1){
    fprintf(stderr,"%s sock->connection(): Error\n",me);
    _in_sock->Print();
    Set_ApplState(STOP_STATE);
    if(_in_sock->get_sockd() != -1) _in_sock->close();
    delete _in_sock;
    _in_sock=0;
    return -1;
  }else{
    fprintf(stderr, "Connected\n");
  }

  // Print Current contents of the data 
  //  fprintf(stderr,"---Configuraion is -------------\n");
  Print();
  //  fprintf(stderr,"--------------------------------\n");

  // Sending Payload 
  int ret;
  if(ret = _in_sock->send((u_int8_t *)&payld, sizeof(Payload)) <= 0){
    if( ret < 0){
      fprintf(stderr, "Send_payload Error: send failed \n");
    }else if(ret ==0){
      fprintf(stderr, "Send_payload Error: Socket already closed\n");
    }
    perror("Send_payload Error");
    Set_ApplState(STOP_STATE);
    if(_in_sock->get_sockd() != -1) _in_sock->close();
    delete _in_sock;
    _in_sock=0;
    return -1;
  }
  fprintf(stderr,"Payload(%d bytes) succesfully sent to server\n", 
	  sizeof(Payload));
  fprintf(stderr,"--------------------------------\n");
  time_t dur = payld.length/payld.clock; //sec
  if(payld.Wait(dur) < 0){
    fprintf(stderr," Connection Closed\n");
    Set_ApplState(STOP_STATE);
    if(_in_sock->get_sockd() != -1) _in_sock->close();
    delete _in_sock;
    _in_sock=0;
    return -1;
  }

  fprintf(stderr, "Sending data (%.3f MB) started\n",
	  (float)byte_sec*sec*1.e-6);

  u_int8_t* wr_adr = rBuf;
  int nsize=0;
  long carry_byte=0;
  long long byte_cnt=0;
  long long remain_byte=0;
  long long total_byte=0;
  for(int isec=0; isec<sec; isec++){
    ipage = isec %2;
    //    fprintf(stderr, "writing to page (%d)\n", ipage);
    //    fprintf(stderr, "%d", ipage);
    if(Check_forWrite(ipage) < 0) { 
      fprintf(stderr,"Check_forWrite Error\n");
      isec=sec; break;}
    w_point = page[ipage];
    remain_byte = byte_sec-carry_byte;
    int try_cnt=0;
    while(remain_byte>0){
      status = _in_sock->recv(wr_adr, Min((unsigned long)remain_byte,
					  sizeof(rBuf)-carry_byte));
      if(status ==0){
	try_cnt++;
	if(try_cnt<10) continue;
	fprintf(stderr,"Connection is closed by Server.\n");
	break;
      }
      if(status < 0){
	if(errno ==0)
	  fprintf(stderr,"Error: Connection wass closed by server\n");
	else
	  fprintf(stderr,"Error in recv() (errno=%d):%s \n", 
		  errno, strerror(errno));
	Set_ApplState(STOP_STATE);
	remain_byte=0;
	break;
      }
      if(status > remain_byte){
	memcpy(w_point,rBuf,remain_byte); w_point+=remain_byte;
	carry_byte = status - remain_byte;
	memmove(rBuf,rBuf+remain_byte, carry_byte);
	wr_adr = rBuf + carry_byte;
	remain_byte=0;
      }else{
	memcpy(w_point,rBuf,status); w_point+=status;
	remain_byte -= status;      
	wr_adr = rBuf;
      }
      byte_cnt  += status;

      //      fprintf(stderr,"Recv %dByte read\n", status);
    }
    try_cnt=0;
    total_byte+=byte_cnt;
    if(byte_cnt >= byte_sec) Write_finsh(ipage);
    else{
      fprintf(stderr,"Remaining byte %Ld not received.\n",remain_byte);
    }
    recvd_bytes[ipage] = byte_cnt; // record of received bytes.
  } // for sec 
  
  fprintf(stderr,"\nTotal (%.3f MB) Received from Server\n",
	  (float)total_byte*1.e-6);
  if(total_byte ==0) Set_ApplState(STOP_STATE);
  //  if(page_state[0] + page_state[1] >0) usleep(1000);
  while(!Check_ApplState(STOP_STATE)) usleep(5000);
  fprintf(stderr,"Connection for Server Closed\n");
   if(_in_sock->get_sockd() != -1) _in_sock->close();
  delete _in_sock;
  _in_sock=0;
  return 0;
}

// Thread 
inline void* Recv_mem_simple::thread_func(void){
  fprintf(stderr,"--Starting Recv_mem_simple thread\n");
  nClientID++; 
  Set_ApplState(NORMAL_STATE);
  if( Setup_data_mem() < 0 ){
    //      ||  recv_mem.Setup_prt_shm() < 0){
    fprintf(stderr,"thread_func teminated due to shmget Error");
  }else{
    long dur = (payld.length / (unsigned long long)payld.clock);
    CommunicateWithServer(dur, srv_ip, srv_port);
    Release_data_mem();
  }
  nClientID--;
    fprintf(stderr,"--Closing Recv_mem_simple thread\n");
}

// inline void Recv_mem_simple::Inc_Rpoint(const u_int32_t d){
//   u_int8_t* end= page[ipage] + mem_data_size;
//   u_int32_t remain = end - r_point;
  
//   if( remain < d ){
//     ipage = (ipage +1) % 2;
//     page_mutex[ipage].lock();
//     page_mutex[ipage].unlock();
//     if(ipage ==1) 
//       r_point += d;
//     else
//       r_point = page[0] + d-remain;
//   }else
//     r_point += d;
  
// }


inline 
const int Read_out(Recv_mem_simple& recv_clt, int isec){
  //  usleep(50);
  int ret=0;
  for(int cnt=0; cnt<isec; cnt++){
    ret = recv_clt.Check_forRead(cnt % 2);
    switch(ret){
    case -1:
      fprintf(stderr,"Memory page (%d)is not available\n", cnt%2);
      return -1;  break;
    case -2:
      fprintf(stderr,"Send out last once\n");
      cnt=isec; break;
    }
    //    u_int8_t* rdadr = recv_clt.Get_Rpoint();
    u_int8_t* rdadr = recv_clt.Get_page_addr(isec%2);
    if(rdadr ==0){
      fprintf(stderr,"Read address is not ready\n");
      return -1;
    }
    fprintf(stderr, "  Read Out data %2d-------\n", cnt);
    for(int j=0; j<4; j++){
      for(int i=0; i<4;i++,rdadr+=4){
	fprintf(stderr, "%x%x%x%x ",
		*rdadr, *(rdadr+1), *(rdadr+2), *(rdadr+3) );
      }
      fputc('\n',stderr);
    }
    //    recv_clt.Read_end(cnt % 2);
  }
  return 0;
}

// inline int Recv_mem_simple::Delete_shm(){
//   int ret = 0;
//   if(ret = shmctl(mem_data_id,IPC_RMID,0)==-1 )
//     perror("shmctl:remove shem_ptr_id");
  
//   if(ret==0){
//     fprintf(stderr,"Shared memory(id=%d) %ld MB is released\n", 
// 	    mem_data_id, mem_data_size/1000000);
//   }
//   return ret;
// }

// inline
void StrUpper(char *str, int n){
  int i;
  for(i=0; i<n; i++) str[i] = toupper((int)str[i]);
}


#endif
