/*
 * Copyright (c) 2003 MIT, Haystack Observatory
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction,including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 */

#ifndef K5_H
#define K5_H

#define _FILE_OFFSET_BITS 64 	/* This has to be defined for LFS */

/* Constant definitions */
#define MAX_SIZE 	5000
#define FWORD		0xFFFFFFFF
#define TIME 		0x1FFFF
#define CHAN 		0x20000
#define FREQ		0x3C0000
#define QUANT		0xC00000
#define SYNC		0xFF000000


#include <TCPSocket.h>
#include <UDPSocket.h>

class K5 
{
	 int parity_table[256];
	 unsigned int bitreverse_table[65536];
	 unsigned int nrzOut;
	 unsigned int   data[4][MAX_SIZE],
		  out,
		  crc16;
	 unsigned char  x, xx[32], iData[6];

	 /* OPTIM_DECLS
	  * These variables declared global for faster access.
	  */
	 unsigned int aaaa;
	 unsigned int dddd;
	 unsigned int datamask;
	 unsigned int outmask;
	 unsigned int xxmask;
	 unsigned int data_d_k;
	 unsigned char *xxp;
	 int fd[4], fdout;
	 TCPSocket _accept_sock;
	 TCPSocket _tcp_out_sock;
	 UDPSocket _udp_out_sock;
	 u_int32_t _tx_buf[352]; // 1408 B.
	 int _tx_buf_idx;
	 int _mjd;
	 bool _udp;

	 /* Options structures */
	 typedef struct {
		  int mjd;
		  char** input_files;
		  int num_files;
		  char* output_files;
		  int channel;
		  int file;
		  char* stream_file;
	 } options_t;

  public:
	 K5(const bool udp, const int& mjd, const string& remote_ip, const int& data_port);
	 ~K5();
	 void init_parity(int* table );
	 void init_bitreverse(unsigned int *table);
	 int slow_parity(unsigned char c);
	 void run();

	 inline unsigned int fast_parity(unsigned char c) {
		  return parity_table[c];
	 }
	 inline unsigned int bitreverse(unsigned int c) {
		  return bitreverse_table[c];
	 }
	 inline int crcc(unsigned char* idata, int len, int mask, int cycl) {
		  /* crc subroutine ripoff */
		  unsigned int istate = 0;
		  int idbit;
		  int q, ich, icb;
		  for (idbit = 1; idbit <= len; idbit++) {
				q = istate & 1;
				ich = (idbit - 1) / 8;
				icb = 7 - (idbit - 1) % 8;
				if ((((idata[ich] >> icb) & 1) ^ q) == 0)
					 istate &= -2;
				else {
					 istate ^= mask;
					 istate |= 1;
				}
				istate = istate >> 1 | (istate & 1) << cycl -1;
		  }
		  return ((int) istate);
	 }
	 inline void WRITE_WORD() {
		  nrzOut ^= out;  
		  _tx_buf[_tx_buf_idx]=nrzOut;
		  if (_tx_buf_idx==351) {
				write(fdout, _tx_buf, 1408);
				_tx_buf_idx=0;
		  } else {
				++_tx_buf_idx;
		  }
	 } 
};

#endif // K5


